/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "ConfigFile.h"

static bool isTrimChar( char x ) {
    return ( x == ' ' || x == '\t' );
}

static void ltrim (std::string &s) {
    s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun(isTrimChar))));
}

static void rtrim (std::string &s) {
    s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun(isTrimChar))).base(), s.end());
}

using namespace std;

ConfigFile::ConfigFile( const std::string& fileName ) {
    ifstream input( fileName.c_str() );
    string currSection( "" );
    string currLine;

    while ( getline( input, currLine ) ) {
        if ( currLine.length() > 0 && currLine[0] != '#' ) {
            string::size_type loc = currLine.find( '=' );
            if ( loc != string::npos ) {
                //This is a variable line
                string varName( currLine, 0, loc );
                string value( currLine, loc+1, string::npos );

                //Trim spaces from the variable name
                ltrim( varName );
                rtrim( varName );

                //Insert the new varible into our maps
                sections[ currSection ][ varName ] = value;

            } else {
                loc = currLine.find( '[' );
                if ( loc != string::npos ) {
                    //This might be a section header line
                    string::size_type endLoc = currLine.find( ']', loc+1 );
                    if ( endLoc != string::npos ) {
                        //It is a section header line
                        currSection = string( currLine, loc + 1, endLoc - loc - 1 );

                        //Trim spaces from the section name
                        ltrim( currSection );
                        rtrim( currSection );
                    }
                }
            }
        }
    }
}

bool ConfigFile::varExists( const std::string& section, const std::string& varName ) {
    SectionMapIter sIter = sections.find( section );
    if ( sIter == sections.end() )
        return false;

    VariableMap& varMap = (*sIter).second;
    VariableMapIter vIter = varMap.find( varName );
    if ( vIter == varMap.end() )
        return false;

    return true;
}

#ifdef _DEBUG
void ConfigFile::doTest() {
    ConfigFile test( "tests/config.ini" );
    assert( test.varExists( "", "variable1" ) );
    assert( test.varExists( "", "variable2" ) );
    assert( test.varExists( "section1", "var2" ) );
    assert( test.varExists( "section1", "variable1" ) );
    assert( !test.varExists( "section1", "var3" ) );
    assert( !test.varExists( "sectionx", "variable1" ) );

    string val;
    test.getVar( "", "variable1", val, string("ERROR") );
    TRACE( val.c_str() );
    test.getVar( "sectionx", "variable1", val, string("Not Found") );
    TRACE( val.c_str() );
    test.getVar( "sectionx", "variable1", val, "Not found" );
    TRACE( val.c_str() );

    int iVal;
    test.getVar( "", "variable2", iVal, 100 );
    TRACE1( "%i", iVal);
    test.getVar( "", "variabl3", iVal, 99 );
    TRACE1( "%i", iVal);
    
    float fVal;
    test.getVar( "section1", "variable1", fVal, 1.0f );
    TRACE1( "%f", fVal);
}
#endif
