/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef ISOBOMB_H_VADE23
#define ISOBOMB_H_VADE23

#pragma warning(disable:4786)

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT   0x0500
#define _WIN32_WINDOWS 0x0410
#define WINVER         0x0400
#include <windows.h>

#define DIRECTINPUT_VERSION 0x0800

#ifdef _DEBUG
#define TRACE(x)                doTrace( __FILE__, __LINE__, x )
#define TRACE1(x, y)            doTrace( __FILE__, __LINE__, x, y )
#define TRACE2(x, y, z)         doTrace( __FILE__, __LINE__, x, y, z )
#define TRACE3(x, y, z, a)      doTrace( __FILE__, __LINE__, x, y, z, a )
#define TRACE4(x, y, z, a, b)   doTrace( __FILE__, __LINE__, x, y, z, a, b )
#else
#define TRACE(x)                ((void)0)
#define TRACE1(x, y)            ((void)0)
#define TRACE2(x, y, z)         ((void)0)
#define TRACE3(x, y, z, a)      ((void)0)
#define TRACE4(x, y, z, a, b)   ((void)0)
#endif

#define sq(x) ( (x) * (x) )

#include <d3d8.h>   // directX8 Header File
#include <d3dx8.h>  // directX8 Helper Functions
#include <Dxerr8.h>
#include <dinput.h> // DirectInput
#include <assert.h>
#include <ctime>
#include <cmath>
#include <cstdio>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <set>
#include <map>
#include <functional>
#include <mmsystem.h>
#include <mmreg.h>
#include <msacm.h>
#include <dsound.h>

#ifndef M_PI
#define M_PI       3.14159265358979323846
#define M_PI_F     3.14159265358979323846f
#endif

#include "sound/SndMan.h"

#include "GameDX.h"
#include "Window.h"
#include "Keyboard.h"
#include "Timer.h"
#include "Point.h"
#include "Rect.h"
#include "Vector.h"
#include "jrnd.h"

// Forward declarations
class Game;
class GameDX;
class Keyboard;
class Window;
class AnimationManager;
class TextureManager;
class SoundSystem;
class ConfigFile;

// Constants
#define DEFAULT_WINDOW_WIDTH    800
#define DEFAULT_WINDOW_HEIGHT   600
#define DEFAULT_WINDOWED        TRUE
#define WINDOW_TITLE            "Super IsoBomb"
#define DISPLAY_FPS             TRUE
#define DRAW_BOUNDING_BOXES     FALSE
#define DIRECTION_UPLEFT        0
#define DIRECTION_UPRIGHT       1
#define DIRECTION_DOWNRIGHT     2
#define DIRECTION_DOWNLEFT      3

// Typedefs
typedef LPDIRECT3DTEXTURE8 Texture;
typedef LPDIRECT3DDEVICE8 GraphicsDevice;

typedef std::vector< Texture > AnimationFrames;
typedef AnimationFrames::iterator AFramesIter;

// Globals

/**
 * Global pointer to the DirectX stuff
 *
 */
extern GameDX* g_dx;

/**
 * Global pointer to the Keyboard
 *
 */
extern Keyboard* g_key;

/**
 * Global pointer to the Window
 *
 */
extern Window* g_win;

/**
 * Global pointer to the Game object
 */
extern Game* g_game;

/**
 * Global pointer to the Animation Manager
 */
extern AnimationManager* g_am;

/**
 * Global pointer to the Texture Manager for map tiles
 */
extern TextureManager* g_tm;

/**
 * Global pointer to the Sound Manager.
 */
extern SoundSystem* g_snd;

/**
 * Global pointer to the configuration.
 */
extern ConfigFile* g_cfg;

/**
 * Global pointer to the Texture Manager for the player HUD
 */
extern TextureManager* g_hudTm;

/**
 * The window width and height.  Global so WndProc can modify.
 *
 */
extern int g_winSizeX; 
extern int g_winSizeY;

/**
 * WinMain
 *
 */
int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPSTR lpCmdLine, int nShowCmd );

/**
 * WndProc
 *
 */
LRESULT CALLBACK GlobalWndProc( HWND hWnd, UINT message, 
							    WPARAM wParam, LPARAM lParam );

/**
 * Outputs to the debug window.  Also outputs the filename and line number
 * where TRACE is called.  Works like printf.
 *
 */
void doTrace( const char* fn, int lineno, const char* msg, ... );

#endif
