/**
 * Super IsoBomb
 *
 * Taken From: The Zen of Direct3D Game Programming (Peter Walsh)
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"

Keyboard::Keyboard() {
    ZeroMemory( &m_KeyBuffer, sizeof(m_KeyBuffer) );
    m_pKeyDev = 0;
    m_bInitialized = FALSE;
}

Keyboard::~Keyboard() {
    if( m_pKeyDev ) {
        m_pKeyDev->Unacquire();
        m_pKeyDev->Release();
    }
}

HRESULT Keyboard::initialize( HWND hWnd ) {
    HRESULT r;

    // Release the device if it has already been created
    if( m_pKeyDev ) {
        m_pKeyDev->Unacquire();
        m_pKeyDev->Release();
    }

    // Create the device for the keyboard
    r = g_dx->getDirectInput()->CreateDevice( GUID_SysKeyboard, &m_pKeyDev, NULL );
    if( FAILED(r) ) {
        TRACE( "Failed to create key device" );
        return E_FAIL;
    }

    // Set the data format for the device
    r = m_pKeyDev->SetDataFormat( &c_dfDIKeyboard );
    if( FAILED(r) ) {
        TRACE( "Unable to set the keyboard data format" );
        return E_FAIL;
    }

    // Set the cooperative level
    r = m_pKeyDev->SetCooperativeLevel( hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE );
    if( FAILED(r) ) {
        TRACE( "Unable to set keyboard cooperative level" );
        return E_FAIL;
    }

    // Acquire the device
    r = m_pKeyDev->Acquire();
    if( FAILED(r) ) {
        TRACE( "Unable to acquire the keyboard" );
        return E_FAIL;
    }

    // Set the initialization flag to true
    m_bInitialized = TRUE;

    return S_OK;
}

bool Keyboard::isKeyDown( int Key ) {
    HRESULT r = 0;

    // Make sure the keyboard has been initialized
    if( !m_bInitialized )
        return false;

    // Get the state of the keyboard into the key buffer
    r = m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
    if( FAILED( r ) ) {
        // If the device is not acquired...
        if( r == DIERR_INPUTLOST ) {
            // ...then reacquire the device
            while( r == DIERR_INPUTLOST ) {
                r = m_pKeyDev->Acquire();
            }

            if( SUCCEEDED( r ) ) {
                m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
            }
            else {
                return false;
            }
        }
        else {
            // ...Otherwise it was some other error
            return false;
        }
    }

    // Check if the key was set
    if( m_KeyBuffer[Key] & 0x80 ) {
        return true;
    }
    else {
        return false;
    }
}