/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#include "IsoBomb.h"
#include "KeyboardController.h"
#include "Virus.h"
#include "Powerup.h"

KeyboardController::KeyboardController() : homingKeyDown(false) {

}

KeyboardController::KeyboardController( Character* c, const KeyboardSettings& ks ) :
    Controller(c), keySettings(ks), homingKeyDown(false) {
}

KeyboardController::~KeyboardController() {

}

void KeyboardController::control( float dt ) {
    // Don't let the player control the character when he's dead
    if( !character->isDead() ) {
        Virus* virus = character->getVirus();
        bool blockBombs = false;
        if( virus != NULL ) {
            blockBombs = (virus->getVirusType() == BOMBARRHEA_VIRUS);
        }

        // Check the movement keys
        if( g_key->isKeyDown(keySettings.up) ) {
            character->move( DIRECTION_UPRIGHT, dt );   
        }
        else if( g_key->isKeyDown(keySettings.right) ) {
            character->move( DIRECTION_DOWNRIGHT, dt );   
        }
        else if( g_key->isKeyDown(keySettings.left) ) {
            character->move( DIRECTION_UPLEFT, dt );   
        }
        else if( g_key->isKeyDown(keySettings.down) ) {
            character->move( DIRECTION_DOWNLEFT, dt );   
        }
        else {
            character->noMove( dt );
            character->stopAnimation();
        }

        // Don't let the character lay bombs if the character has
        // a virus which blocks bombs keys.
        if( !blockBombs ) {
            // Check the bomb key
            if( g_key->isKeyDown(keySettings.bomb) ) {
                character->chargeThrow( dt );
            }
            else if( character->isCharging() ) {
                character->throwBomb();
            }

            // Only lay a homing bomb on key up
            if( homingKeyDown ) {
                if( !g_key->isKeyDown(keySettings.homing) ) {
                    character->homingBomb();
                    homingKeyDown = false;
                }
            }
            else {
                homingKeyDown = g_key->isKeyDown( keySettings.homing );
            }
        }
    }
}