/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _KEYBOARDCONTROLLER_H_
#define _KEYBOARDCONTROLLER_H_

#include "Controller.h"
#include "Character.h"

/**
 * Stores a mapping for each of the game's action buttons.
 */
struct KeyboardSettings {
    int left;
    int right;
    int up;
    int down;
    int bomb;
    int homing;
};

/**
 * A controller that uses input received from the keyboard
 * to control a character.
 */
class KeyboardController : public Controller {

public:

    /**
     * Default constructor
     */
    KeyboardController();

    /**
     * Constructor
     *
     * @param c                 the character to control
     * @param ks                the keyboard settings to use
     */
    KeyboardController( Character* c, const KeyboardSettings& ks );

    /**
     * Destructor
     */
    ~KeyboardController();

    /**
     * Control the attached character.
     *
     * @param dt                the amount of time since the last frame
     */
    virtual void control( float dt );

private:

    /**
     * The keyboard settings to use.
     */
    KeyboardSettings keySettings;

    /**
     * Used to tell when the homing key was let up.
     */
    bool homingKeyDown;

};

#endif