/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _RECT_H_
#define _RECT_H_

#include "IsoBomb.h"

/**
 * The Rect class represents a rectangular section, usually used to represent
 * rectangles in world coordinates.
 */
class Rect {
public:
    float x, y, w, h;

    Rect() : x(0), y(0), w(0), h(0) {}
    Rect(float x2, float y2, float w2, float h2) : x(x2), y(y2), w(w2), h(h2) {}

    void init(float x2, float y2, float w2, float h2) {
        x = x2; y = y2; w = w2; h = h2;
    }

    float getRight() const {  //inclusive point
        return x+w;
    }

    float getBottom() const { //inclusive point
        return y+h;
    }

    Point getUpperLeft() const {
        return Point(x, y);
    }

    Point getLowerRight() const {
        return Point( x+w, y+h );
    }

    Point getUpperRight() const {
        return Point( x+w, y );
    }

    Point getLowerLeft() const {
        return Point( x, y+h );
    }

    Point getCenter() const {
        return Point(x+w/2.0f, y+h/2.0f);
    }

    void setCenter( const Point& cent ) {
        x = cent.x - (w / 2);
        y = cent.y - (h / 2);
    }

    /**
    * @returns true if the point is in or on the border of this rect.
    */
    bool isCollision(const Point& other) const {
        return ((other.x <= x+w) && (other.y <= y+h) &&
            (other.x >= x) && (other.y >= y));
    }

    /**
    * @returns true if the two rectangles collide.
    */
    bool isCollision(const Rect& other) const {
        return ((other.x <= x+w) && (other.y <= y+h) &&
            (other.x+other.w >= x) && (other.y+other.h >= y));
    }

    /**
    * Finds distance based on midpoints.
    */
    float distTo(const Rect& other) const {
        return (float)(fabs(sqrt(sq((other.x+other.w/2.0f)-(x+w/2.0f)) + 
                            sq((other.y+other.h/2.0f)-(y+h/2.0f)))));
    }

protected:
private:
};

#endif
