/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef RESPAWNENTITY_H_PQM302
#define RESPAWNENTITY_H_PQM302

#include "GameLogic.h"

class Entity;

/**
 * The RespawnEntity class waits for the given amount of time.  After the time
 * has passed, the Entity given to the object is inserted into the game's list
 * of Entities.
 *
 * Respawn Entity is not default-constructable or copyable.
 */
class RespawnEntity : public GameLogic {
public:
    /**
     * Constructs a new RespawnEntity.  Adds the passed Entity ent to the game
     * after timeToRespawn has passed.  After ent is back into the game, the
     * update method will return true.
     */
    RespawnEntity( Entity* ent, float timeToRespawn );

    /**
     * Cleans up this object.  If this object is destructed, it does delete
     * the passed in Entity.  This should only happen if the program exits
     * before the respawn event takes place.
     */
    virtual ~RespawnEntity();

    /**
     * Waits to respawn this object's Entity.  After the Entity has been added
     * back into the game, update returns true.
     */
    bool update( float dt );

private:
    RespawnEntity();
    RespawnEntity( const RespawnEntity& other );
    RespawnEntity& operator = ( const RespawnEntity& rhs );

    float timeLeft;
    Entity* ent;
};

#endif
