/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef _THROWNBOMB_H_
#define _THROWNBOMB_H_

#include "Bomb.h"
#include "IsoBomb.h"
#include "Character.h"

/**
 * A bomb that has been thrown by the player.
 */
class ThrownBomb : public Bomb {

public:

    /**
     * Default constructor
     */
    ThrownBomb();

    /**
     * Constructor
     *
     * @param initPos                   initial map position
     * @param initDir                   initial direction
     * @param size                      blast size
     * @param delay                     detonation delay
     * @param power                     amount of power used to throw the bomb
     * @param c                         character who threw this bomb
     */
    ThrownBomb( const Rect& initPos, int initDir, float size, float delay, 
                float power, Character* c );

    /**
     * Destructor
     */
    virtual ~ThrownBomb();

    /**
     * Update the bomb's logic.
     *
     * @param dt                        the amount of time since the last frame
     * @returns                         true if this bomb has blown up
     */
    virtual bool update( float dt );

    /**
     * Draw the bomb on the screen using the specified anchor point.
     *
     * @param anchor                    the bomb's anchor point
     */
    virtual void draw( const Point& anchor );
    

	/**
	 *	Handles the collision between 2 entities by finding what type of
	 *	entities they are and then handling the different types of collisions;
	 *
	 *  @param otherEntity       the entity to check collision with
	 */
	virtual void doCollision( Entity* otherEntity );

private:

    /**
     * Vector used to update the bomb's height and position.
     */
    Vector vect;

    /**
     * The character who threw this bomb
     */
    Character* owner;

    /**
     * True if this bomb has landed
     */
    bool hasLanded;

};

#endif