/**
 * Super IsoBomb
 *
 * Authors: Jim Clase, Jonathan Hilliker, Jason Winnebeck
 */

#ifndef TILELOCATION_H_ERM423
#define TILELOCATION_H_ERM423

/**
 * A TileLocation object represents a tile location in a diamond isomap that
 * can be moved around in any of the 8 "screen" directions.  TileLocations
 * can be copied.
 *
 * The TileLocation class is implemented in the header as its implementation
 * is trivial from both a programmer and computational perspective.  It is
 * also used in the map's render loop and thus TileLocation makes a good
 * canidate for inlining.
 */
class TileLocation {
public:
    /**
     * The default ctor creates a location at 0,0.
     */
    TileLocation() : x(0), y(0) {}

    /**
     * Copy constructor works as expected.
     */
    TileLocation( const TileLocation& o ) : x(o.x), y(o.y) {}

    /**
     * Constructor for specifing initial location.
     */
    TileLocation( int x, int y ) : x(x), y(y) {}

    /**
     * Destructor.
     */
    ~TileLocation() {}

    /**
     * Returns the X location.
     */
    int getX() const { return x; }

    /**
     * Returns the Y location.
     */
    int getY() const { return y; }

    /**
     * Moves the TileLocation one tile "north" (as defined by the "screen").
     */
    void north() { --x; --y; }

    /**
     * Moves the TileLocation one tile "northeast" (as defined by the "screen").
     */
    void northeast() { --y; }

    /**
     * Moves the TileLocation one tile "east" (as defined by the "screen").
     */
    void east() { ++x; --y; }

    /**
     * Moves the TileLocation one tile "southeast" (as defined by the "screen").
     */
    void southeast() { ++x; }

    /**
     * Moves the TileLocation one tile "south" (as defined by the "screen").
     */
    void south() { ++x; ++y; }

    /**
     * Moves the TileLocation one tile "southwest" (as defined by the "screen").
     */
    void southwest() { ++y; }

    /**
     * Moves the TileLocation one tile "west" (as defined by the "screen").
     */
    void west() { --x; ++y; }

    /**
     * Moves the TileLocation one tile "northwest" (as defined by the "screen").
     */
    void northwest() { --x; }

    /**
     * Assignment operator works as expected.
     */
    TileLocation& operator= ( const TileLocation& rhs ) {
        x = rhs.x;
        y = rhs.y;
        return *this;
    }

    /**
     * Equality operator works as expected.
     */
    bool operator== ( const TileLocation& rhs ) {
        return ( x == rhs.x && y == rhs.y );
    }

private:
    int x;
    int y;
};

#endif
